SystemOrganization addCategory: #'StashShell-Pharo-Morphic'!
SystemOrganization addCategory: #'StashShell-Pharo-Transcript'!

PluggableTextMorph subclass: #StashShellWindowMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StashShell-Pharo-Morphic'!

!StashShellWindowMorph commentStamp: '<historical>' prior: 0!
I am a simple teletype shell session morph, providing a view on an instance of CommandShell.!

!StashShellWindowMorph methodsFor: 'menu commands' stamp: 'dtl 5/27/2002 08:57'!
accept

	self model cr; flush; processCommand: self commandLineInput asString
! !

!StashShellWindowMorph methodsFor: 'updating' stamp: 'dtl 1/21/2007 13:02'!
appendEntry

	| str |
	"Append the text in the model's writeStream to the editable text. "
	textMorph asText size > model characterLimit ifTrue:
		["Knock off first half of text"
		self selectInvisiblyFrom: 1 to: textMorph asText size // 2.
		self replaceSelectionWith: Text new].
	self selectInvisiblyFrom: textMorph asText size + 1 to: textMorph asText size.
	str := model contents.
	(str size > 0) ifTrue:
		[self replaceSelectionWith: (Text
			string: str
			attribute: (TextFontChange fontNumber: self textStyle defaultFontIndex)).
		self selectInvisiblyFrom: textMorph asText size + 1 to: textMorph asText size.
		model reset]

! !

!StashShellWindowMorph methodsFor: 'command input' stamp: 'dtl 3/18/2001 18:17'!
commandLineInput

	^ (self text copyFrom: self positionAfterPromptString to: self text size) asString.

! !

!StashShellWindowMorph methodsFor: 'command input' stamp: 'dtl 4/7/2001 12:21'!
positionAfterPromptString
	"Answer the index of the first character after the last prompt string in my text. If
	not found, then assume that the contents of the text are all intended to be command
	input."

	| t loc |
	t := self text.
	(1 to: (t size - model promptString size))
		reverseDo: [:i |
			((loc := t findString: model promptString startingAt: i) ~= 0)
				ifTrue: [^ loc + model promptString size]].
	^ 1
! !

!StashShellWindowMorph methodsFor: 'model access' stamp: 'DaleHenrichs 12/11/2010 12:56'!
setText: aText
	scrollBar setValue: 0.0.
	textMorph
		ifNil: [textMorph := StashShellTtyTextMorphForEditView new
						contents: aText wrappedTo: self innerBounds width-6.
				textMorph setEditView: self.
				textMorph setTextStyle: self textStyle.
				scroller addMorph: textMorph]
		ifNotNil: [textMorph newContents: aText].
	self hasUnacceptedEdits: false.
	self setScrollDeltas.! !

!StashShellWindowMorph methodsFor: 'model access' stamp: 'dtl 11/18/2009 22:20'!
textStyle
	"A fixed width font for the text morph"

	^ (TextStyle named: 'DefaultFixedTextStyle')
		ifNil: [TextStyle default]! !

!StashShellWindowMorph methodsFor: 'updating' stamp: 'dtl 6/3/2001 19:02'!
update: something

	(something == #doCommand)
		ifTrue:
			[^ self accept].
	(something == #exit)
		ifTrue:
			[^ self owner release; delete].
	^ super update: something
! !

!StashShell classSide methodsFor: '*stashshell-pharo' stamp: 'DaleHenrichs 12/11/2010 10:00'!
open
	"self open"

	^ self default open! !

!StashShell methodsFor: '*stashshell-pharo' stamp: 'DaleHenrichs 12/9/2010 10:14'!
doEndOfFile
	"EOF character detected."
! !

!StashShell methodsFor: '*stashshell-pharo' stamp: 'DaleHenrichs 12/9/2010 10:15'!
filterAndForwardEvent: keyboardEvent 
	"Filter keyboardEvent, taking special action if needed. If a child process
	is active, forward keyboardEvent to the child and answer nil. Otherwise
	answer keyboardEvent."

	^ keyboardEvent! !

!StashShell methodsFor: '*stashshell-pharo' stamp: 'DaleHenrichs 12/9/2010 10:06'!
open

	| tty |
	tty := StashShellTranscriptStream stashShell: self.
	tty open.
	^ tty

! !

TranscriptStream subclass: #StashShellTranscriptStream
	instanceVariableNames: 'cliShell'
	classVariableNames: 'StashAccessSema'
	poolDictionaries: ''
	category: 'StashShell-Pharo-Transcript'!

!StashShellTranscriptStream classSide methodsFor: 'as yet unclassified' stamp: 'DaleHenrichs 12/9/2010 09:27'!
newTranscript: aTextCollector 

	self shouldNotImplement! !

!StashShellTranscriptStream classSide methodsFor: 'as yet unclassified' stamp: 'DaleHenrichs 12/9/2010 09:28'!
openMorphicTranscript
	self shouldNotImplement! !

!StashShellTranscriptStream classSide methodsFor: 'initialization' stamp: 'DaleHenrichs 12/9/2010 09:29'!
registerInFlapsRegistry
! !

!StashShellTranscriptStream classSide methodsFor: 'instance creation' stamp: 'DaleHenrichs 12/9/2010 10:08'!
stashShell: cliShell

	| ttyDisplay |
	ttyDisplay := self new cliShell: cliShell.
	^ ttyDisplay
! !

!StashShellTranscriptStream classSide methodsFor: 'initialization' stamp: 'DaleHenrichs 12/9/2010 09:28'!
unload
! !

!StashShellTranscriptStream classSide methodsFor: 'window color' stamp: 'DaleHenrichs 12/9/2010 09:28'!
windowColorSpecification
	"Answer a WindowColorSpec object that declares my preference"

	^ WindowColorSpec classSymbol: self name wording: 'Stash Shell' brightColor: #lightOrange pastelColor: #paleOrange helpMessage: 'The stash shell'! !

!StashShellTranscriptStream methodsFor: 'toolbuilder' stamp: 'DaleHenrichs 12/9/2010 09:31'!
buildWith: builder
	| windowSpec textSpec |
	windowSpec := builder pluggableWindowSpec new.
	windowSpec model: self.
	windowSpec label: self label.
	windowSpec children: OrderedCollection new.

	textSpec := builder pluggableTextSpec new.
	textSpec 
		model: self;
		menu: #codePaneMenu:shifted:;
		frame: (0@0corner: 1@1).
	windowSpec children add: textSpec.

	^builder build: windowSpec! !

!StashShellTranscriptStream methodsFor: 'input character filtering' stamp: 'DaleHenrichs 12/9/2010 10:12'!
checkSttyForEvent: keyboardEvent
	"Check for interrupt characters and such. Consume interrupt character and
	answer nil, otherwise answer keyboardEvent."

	(self isInterruptEvent: keyboardEvent)
		ifTrue:
			[self handleInterruptCharacterEvent.
			^ nil].
	(self isEndOfFileEvent: keyboardEvent)
		ifTrue:
			[self cliShell doEndOfFile.
			^ nil].
	^ keyboardEvent! !

!StashShellTranscriptStream methodsFor: 'accessing' stamp: 'DaleHenrichs 12/9/2010 10:03'!
cliShell
	"Answer the value of cliShell"

	^ cliShell ifNil: [cliShell := StashShell new]! !

!StashShellTranscriptStream methodsFor: 'accessing' stamp: 'DaleHenrichs 12/9/2010 10:03'!
cliShell: aShell

	cliShell := aShell! !

!StashShellTranscriptStream methodsFor: 'input character filtering' stamp: 'DaleHenrichs 12/9/2010 10:11'!
filterAndForwardEvent: keyboardEvent 
	"Filter keyboardEvent, taking special action if needed. If a child process
	is active, forward keyboardEvent to the child and answer nil. Otherwise
	answer keyboardEvent."

	^ (self checkSttyForEvent: keyboardEvent)
		ifNotNil: [self cliShell filterAndForwardEvent: keyboardEvent]! !

!StashShellTranscriptStream methodsFor: 'input character filtering' stamp: 'DaleHenrichs 12/9/2010 10:12'!
handleInterruptCharacterEvent
	"Interrupt character detected, do interrupt stuff."

! !

!StashShellTranscriptStream methodsFor: 'input character filtering' stamp: 'DaleHenrichs 12/9/2010 10:13'!
isEndOfFileEvent: keystrokeEvent 
	"A <ctl>d event, represented either as character value 4, or as $d with
	the control or meta key. The actual representation varies on different
	versions of Squeak."

	^ keystrokeEvent keyValue == 4
		or: [keystrokeEvent keyCharacter = $d
				and: [keystrokeEvent controlKeyPressed]]! !

!StashShellTranscriptStream methodsFor: 'input character filtering' stamp: 'DaleHenrichs 12/9/2010 10:13'!
isInterruptEvent: keystrokeEvent 
	"A <ctl>c event, represented either as character value 3, or as $c with
	the control or meta key. The actual representation varies on different
	versions of Squeak."

	true ifTrue: [ ^false ].
	^ keystrokeEvent keyValue == 3
		or: [keystrokeEvent keyCharacter = $c
				and: [keystrokeEvent controlKeyPressed]]! !

!StashShellTranscriptStream methodsFor: 'printing' stamp: 'DaleHenrichs 12/9/2010 09:30'!
isSelfEvaluating
	^false! !

!StashShellTranscriptStream methodsFor: 'accessing' stamp: 'DaleHenrichs 12/9/2010 09:29'!
label

	^'Stash Shell'! !

!StashShellTranscriptStream methodsFor: 'initialization' stamp: 'DaleHenrichs 12/9/2010 09:30'!
open
	| openCount |
	openCount := 0.
	self dependents
		do: [:d | (d isSystemWindow)
				ifTrue: [openCount := openCount + 1]].
	openCount = 0
		ifTrue: [self openLabel: self label]
		ifFalse: [self openLabel: self label, ' #' , (openCount + 1) printString]! !

!StashShellTranscriptStream methodsFor: 'initialization' stamp: 'DaleHenrichs 12/9/2010 09:30'!
openAsMorph
	"Answer a morph viewing this transcriptStream"

	^ (self openAsMorphLabel: self label) extent: self initialExtent! !

!StashShellTranscriptStream methodsFor: 'initialization' stamp: 'DaleHenrichs 12/11/2010 12:56'!
openAsMorphLabel: labelString
	"Build a morph viewing this transcriptStream"

	| window textMorph |
	window := (SystemWindow labelled: labelString) model: self.
	textMorph := StashShellWindowMorph
		on: self
		text: nil
		accept: nil
		readSelection: nil
		menu: #codePaneMenu:shifted:.
	textMorph acceptOnCR: true.
	window addMorph: textMorph frame: (0 @ 0 corner: 1 @ 1).
	self prompt.
	^ window! !

!StashShellTranscriptStream methodsFor: 'evaluation' stamp: 'DaleHenrichs 12/9/2010 16:30'!
processCommand: aCommandString
	"Evaluate aCommandString in a separate Smalltalk process. This permits the
	user interface to remain responsive."

	(self cliShell processCommand: aCommandString)
		ifNotNil: [ :output | 
			| str |
			(str := output asString) notEmpty
				ifTrue: [ 
					self
						show: output asString;
						cr ] ].
	self prompt! !

!StashShellTranscriptStream methodsFor: 'command prompt' stamp: 'DaleHenrichs 12/9/2010 10:31'!
prompt

	self show: self cliShell promptString! !

!StashShellTranscriptStream methodsFor: 'command prompt' stamp: 'DaleHenrichs 12/9/2010 10:16'!
promptString
	"Dependents call this when restoring the prompt string display"

	^self cliShell promptString! !

!StashShellTranscriptStream methodsFor: 'evaluation' stamp: 'DaleHenrichs 12/9/2010 10:01'!
scheduleToEvaluate: aBlock
	"Evaluate aBlock, typically to create a new scheduled window. Make it work in
	both Morphic."

	WorldState addDeferredUIMessage: aBlock! !

!StashShellTranscriptStream methodsFor: 'private' stamp: 'DaleHenrichs 12/9/2010 09:32'!
semaphore
	^StashAccessSema ifNil:[StashAccessSema := Semaphore forMutualExclusion]! !

!StashShellTranscriptStream methodsFor: 'updating' stamp: 'DaleHenrichs 12/9/2010 10:25'!
update: event

	event == #prompt	"display $PS1"
		ifTrue: [^ self prompt].
	event == #alternatePrompt	"display $PS2"
		ifTrue: [^ self promptTwo].
	event == #restorePrompt	"restore last prompt display"
		ifTrue: [^ self prompt].
	event == #exit
		ifTrue: [^ self close].
	event == #clearText
		ifTrue: [^ self clear].
	event == #interruptCharacter
		ifTrue: [^ self handleInterruptCharacterEvent].
	event == #relabel
		ifTrue: [^ self changed: event].
	self error: ['unexpected event']! !

TextMorphForEditView subclass: #StashShellTtyTextMorphForEditView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StashShell-Pharo-Morphic'!

!StashShellTtyTextMorphForEditView commentStamp: '<historical>' prior: 0!
I add a keyboard hook to a TextMorphForEditView to allow filtering of keystrokes for a simple tty terminal emulator.!

!StashShellTtyTextMorphForEditView methodsFor: 'editing' stamp: 'dtl 11/2/2007 05:18'!
keyStroke: evt

	(self editView model filterAndForwardEvent: evt)
		ifNotNil: [^ super keyStroke: evt]
! !
